@@ -23,12 +23,15 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include "common/WWW.h"
 #include "common/XMSession.h"
 #include "helpers/Log.h"
+#include "helpers/Time.h"
 #include "helpers/VExcept.h"
 #include "xmoto/Game.h"
 #include "xmoto/GameText.h"
+#include "xmoto/input/Input.h"
+#include "xmoto/input/InputLegacy.h"
 #include <sstream>
 
-#define XMDB_VERSION 36
+#define XMDB_VERSION 37
 #define DB_MAX_SQL_RUNTIME 0.25
 #define DB_BUSY_TIMEOUT 60000 // 60 seconds
 
@@ -47,8 +50,10 @@ void xmDatabase::openIfNot(const std::string &i_dbFileUTF8) {
   if (m_db != NULL)
     return;
 
+  std::string dbFile = XMFS::getUserDirUTF8(FDT_DATA) + "/" + i_dbFileUTF8;
+
   // LogDebug("openDB(%X)", this);
-  if (sqlite3_open(i_dbFileUTF8.c_str(), &m_db) != 0) {
+  if (sqlite3_open(dbFile.c_str(), &m_db) != 0) {
     throw Exception("Unable to open the database (" + i_dbFileUTF8 +
                     ") : " + sqlite3_errmsg(m_db));
   }
@@ -89,7 +94,22 @@ void xmDatabase::preInitForProfileLoading(const std::string &i_dbFileUTF8) {
   }
 }
 
-void xmDatabase::init(const std::string &i_dbFileUTF8,
+void xmDatabase::backupXmDb(const std::string &dbFile) {
+  std::ostringstream backupName;
+  backupName << "xm.v" << m_openingVersion << "." << currentDateTime() << ".db";
+
+  std::string outputPath;
+  if (!XMFS::copyFile(FDT_DATA,
+                      dbFile,
+                      "Backups/" + backupName.str(),
+                      outputPath,
+                      true)) {
+    throw Exception("Failed to copy database file");
+  }
+  LogInfo("Database backed up to: %s", outputPath.c_str());
+}
+
+bool xmDatabase::init(const std::string &i_dbFileUTF8,
                       const std::string &i_profile,
                       const std::string &i_gameDataDir,
                       const std::string &i_userDataDir,
@@ -130,8 +150,27 @@ void xmDatabase::init(const std::string &i_dbFileUTF8,
   }
 
   if (m_openingVersion < XMDB_VERSION) {
+    if (m_openingVersion != 0) {
+      LogInfo("Backing up XmDb");
+
+      try {
+        backupXmDb(i_dbFileUTF8);
+      } catch (Exception &e) {
+        LogError("Failed to backup database:");
+        LogError(e.getMsg().c_str());
+        LogError("Bailing out..");
+
+        if (m_db != NULL) {
+          sqlite3_close(m_db);
+          m_db = NULL;
+        }
+
+        return false;
+      }
+    }
+
     LogInfo(
-      "Update XmDb version from %i to %i", m_openingVersion, XMDB_VERSION);
+      "Updating XmDb version from %i to %i", m_openingVersion, XMDB_VERSION);
 
     if (i_interface != NULL) {
       i_interface->updatingDatabase(GAMETEXT_DB_UPGRADING, 0);
@@ -191,6 +230,8 @@ void xmDatabase::init(const std::string &i_dbFileUTF8,
       }
     }
   }
+
+  return true;
 }
 
 void xmDatabase::updateXMDirectories(const std::string &i_oldGameDataDir,
@@ -265,6 +306,7 @@ void xmDatabase::init(const std::string &i_dbFile, bool i_readOnly) {
 xmDatabase::~xmDatabase() {
   if (m_db != NULL) {
     sqlite3_close(m_db);
+    m_db = NULL;
   }
 }
 
@@ -967,6 +1009,21 @@ void xmDatabase::upgradeXmDbToVersion(int i_fromVersion,
         throw Exception("Unable to update xmDb from 35: " + e.getMsg());
       }
 
+    case 36:
+      try {
+        auto config = GameApp::instance()->getUserConfig();
+
+        Input::instance()->loadConfig(config, this, i_profile);
+        InputSDL12Compat::remap();
+        Input::instance()->saveConfig(config, this, i_profile);
+
+        updateXmDbVersion(37, i_interface);
+
+        Logger::deleteLegacyLog();
+      } catch (Exception &e) {
+        throw Exception("Unable to update xmDb from 36: " + e.getMsg());
+      }
+
       // next
   }
 }
