@@ -24,7 +24,8 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 #include "Game.h"
 #include "GameText.h"
-#include "Input.h"
+#include "input/Input.h"
+#include "input/Joystick.h"
 #include "PhysSettings.h"
 #include "Sound.h"
 #include "common/VFileIO.h"
@@ -32,9 +33,11 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include "helpers/Environment.h"
 #include "helpers/Log.h"
 #include "helpers/Random.h"
+#include "helpers/Time.h"
 
 #include "Credits.h"
 #include "GeomsManager.h"
+#include "LuaLibBase.h"
 #include "Replay.h"
 #include "SysMessage.h"
 #include "XMDemo.h"
@@ -56,6 +59,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include "states/StatePreplayingReplay.h"
 #include "states/StateReplaying.h"
 #include "states/StateWaitServerInstructions.h"
+#include "states/StatePlayingLocal.h"
 
 #include "thread/UpgradeLevelsThread.h"
 
@@ -96,12 +100,7 @@ BOOL LibraryOK(STRPTR libname, ULONG version, ULONG revision) {
 }
 #endif
 
-#if defined(WIN32)
-int SDL_main(int nNumArgs, char **ppcArgs) {
-#else
 int main(int nNumArgs, char **ppcArgs) {
-#endif
-
 #ifdef __amigaos4__
   /* checking for MiniGL 2.0 */
 
@@ -129,7 +128,7 @@ int main(int nNumArgs, char **ppcArgs) {
     snprintf(cBuf,
              1024,
              "Fatal exception occurred: %s\n"
-             "Consult the file xmoto.log for more information about what\n"
+             "Consult the logs/xmoto.log file for more information about what\n"
              "might has occurred.\n",
              e.getMsg().c_str());
     MessageBox(NULL, cBuf, "X-Moto Error", MB_OK | MB_ICONERROR);
@@ -223,14 +222,13 @@ void GameApp::run_load(int nNumArgs, char **ppcArgs) {
   if (v_xmArgs.isOptConfigPath()) {
     XMFS::init("xmoto",
                "xmoto.bin",
-               "xmoto.log",
                v_xmArgs.isOptServerOnly() == false,
                v_xmArgs.getOpt_configPath_path());
   } else {
     XMFS::init(
-      "xmoto", "xmoto.bin", "xmoto.log", v_xmArgs.isOptServerOnly() == false);
+      "xmoto", "xmoto.bin", v_xmArgs.isOptServerOnly() == false);
   }
-  Logger::init("xmoto.log");
+  Logger::init();
 
   /* c xmoto files */
   if (v_xmArgs.isOptBuildQueries()) {
@@ -249,10 +247,10 @@ void GameApp::run_load(int nNumArgs, char **ppcArgs) {
   }
 
   XMSession::setDefaultInstance("live");
-  XMSession::instance()->load(
+  XMSession::instance()->loadConfig(
     m_userConfig); /* overload default session by userConfig */
 
-  XMSession::instance()->load(
+  XMSession::instance()->loadArgs(
     &v_xmArgs); /* overload default session by xmargs     */
 
   Logger::setVerbose(
@@ -261,11 +259,10 @@ void GameApp::run_load(int nNumArgs, char **ppcArgs) {
                    false); /* apply log activ mode */
 
   LogInfo(std::string("X-Moto " + XMBuild::getVersionString(true)).c_str());
-  if (SwapEndian::bigendien) {
-    LogInfo("Systeme is bigendien");
-  } else {
-    LogInfo("Systeme is littleendien");
-  }
+  LogInfo("Started at %s", iso8601Date().c_str());
+  auto endianness = SwapEndian::bigendien ? "big" : "little";
+  LogInfo("System is %s-endian", endianness);
+
   LogInfo("User data   directory: %s", XMFS::getUserDir(FDT_DATA).c_str());
   LogInfo("User config directory: %s", XMFS::getUserDir(FDT_CONFIG).c_str());
   LogInfo("User cache  directory: %s", XMFS::getUserDir(FDT_CACHE).c_str());
@@ -314,19 +311,19 @@ void GameApp::run_load(int nNumArgs, char **ppcArgs) {
     pDb->setTrace(XMSession::instance()->sqlTrace());
   }
 
-  XMSession::instance("file")->load(m_userConfig);
+  XMSession::instance("file")->loadConfig(m_userConfig);
   XMSession::instance("file")->loadProfile(
     XMSession::instance("file")->profile(), pDb);
   (*XMSession::instance()) = (*XMSession::instance("file"));
 
-  XMSession::instance()->load(
+  XMSession::instance()->loadArgs(
     &v_xmArgs); /* overload default session by xmargs */
   // enable propagation only after overloading by command args
   XMSession::enablePropagation("file");
 
   LogInfo("SiteKey: %s", XMSession::instance()->sitekey().c_str());
 
-#ifdef USE_GETTEXT
+#if USE_GETTEXT
   std::string v_locale = Locales::init(XMSession::instance()->language());
   LogInfo("Locales set to '%s' (directory '%s')",
           v_locale.c_str(),
@@ -345,8 +342,8 @@ void GameApp::run_load(int nNumArgs, char **ppcArgs) {
   }
 #endif
 
-  _InitWin(
-    v_useGraphics); // initwin inits sdl (and only sdl if initwin is false)
+  // _InitWin initializes SDL (and only SDL if the argument is false)
+  _InitWin(v_useGraphics);
 
   /* drawlib */
   if (v_useGraphics) {
@@ -365,37 +362,44 @@ void GameApp::run_load(int nNumArgs, char **ppcArgs) {
     drawLib->setDontUseGLExtensions(XMSession::instance()->glExts() == false);
     drawLib->setDontUseGLVOBS(XMSession::instance()->glVOBS() == false);
 
-    LogInfo("Wanted resolution: %ix%i (%i bpp)",
+    LogInfo("Wanted resolution: %ix%i",
             XMSession::instance()->resolutionWidth(),
-            XMSession::instance()->resolutionHeight(),
-            XMSession::instance()->bpp());
+            XMSession::instance()->resolutionHeight());
     drawLib->init(XMSession::instance()->resolutionWidth(),
                   XMSession::instance()->resolutionHeight(),
-                  XMSession::instance()->bpp(),
                   XMSession::instance()->windowed());
     /* drawlib can change the final resolution if it fails, then, reinit session
      * one's */
     XMSession::instance()->setResolutionWidth(drawLib->getDispWidth());
     XMSession::instance()->setResolutionHeight(drawLib->getDispHeight());
-    XMSession::instance()->setBpp(drawLib->getDispBPP());
     XMSession::instance()->setWindowed(drawLib->getWindowed());
-    LogInfo("Resolution: %ix%i (%i bpp)",
+    LogInfo("Resolution: %ix%i",
             XMSession::instance()->resolutionWidth(),
-            XMSession::instance()->resolutionHeight(),
-            XMSession::instance()->bpp());
-    /* */
+            XMSession::instance()->resolutionHeight());
+
+    /* set initial focus state */
+    uint32_t wflags = SDL_GetWindowFlags(GameApp::instance()->getDrawLib()->getWindow());
+    m_hasKeyboardFocus = wflags & SDL_WINDOW_INPUT_FOCUS;
+    m_hasMouseFocus    = wflags & SDL_WINDOW_MOUSE_FOCUS;
+    m_isIconified      = wflags & SDL_WINDOW_HIDDEN;
+  } else {
+    // Doesn't matter anyway
+    m_hasKeyboardFocus = m_hasMouseFocus = m_isIconified = false;
   }
 
   /* init the database (fully, including upgrades) */
-  pDb->init(DATABASE_FILE,
-            XMSession::instance()->profile() == ""
-              ? std::string("")
-              : XMSession::instance()->profile(),
-            XMFS::getSystemDataDir(),
-            XMFS::getUserDir(FDT_DATA),
-            XMFS::binCheckSum(),
-            v_xmArgs.isOptNoDBDirsCheck() == false,
-            this);
+  if (!pDb->init(DATABASE_FILE,
+                 XMSession::instance()->profile() == ""
+                   ? std::string("")
+                   : XMSession::instance()->profile(),
+                 XMFS::getSystemDataDir(),
+                 XMFS::getUserDir(FDT_DATA),
+                 XMFS::binCheckSum(),
+                 v_xmArgs.isOptNoDBDirsCheck() == false,
+                 this)) {
+    quit();
+    return;
+  }
 
   if (v_useGraphics) {
     // allocate the statemanager instance so that if it fails, it's not in a
@@ -415,7 +419,7 @@ void GameApp::run_load(int nNumArgs, char **ppcArgs) {
 
   // network requires the input information (to display the chat command)
   if (v_useGraphics) {
-    InputHandler::instance()->init(m_userConfig,
+    Input::instance()->init(m_userConfig,
                                    pDb,
                                    XMSession::instance()->profile(),
                                    XMSession::instance()->enableJoysticks());
@@ -508,29 +512,8 @@ void GameApp::run_load(int nNumArgs, char **ppcArgs) {
     m_PlaySpecificReplay = v_xmArgs.getOpt_replay_file();
   }
   if (v_xmArgs.isOptDemo()) {
-    /* demo : download the level and the replay
-       load the level as external,
-       play the replay
-     */
     try {
-      m_xmdemo = new XMDemo(v_xmArgs.getOpt_demo_file());
-      LogInfo("Loading demo file %s\n", v_xmArgs.getOpt_demo_file().c_str());
-
-      _UpdateLoadingScreen(GAMETEXT_DLLEVEL);
-      m_xmdemo->getLevel(XMSession::instance()->proxySettings());
-      _UpdateLoadingScreen(GAMETEXT_DLREPLAY);
-      m_xmdemo->getReplay(XMSession::instance()->proxySettings());
-
-      try {
-        LevelsManager::instance()->addExternalLevel(
-          m_xmdemo->levelFile(),
-          xmDatabase::instance("main"),
-          v_xmArgs.isOptServerOnly());
-      } catch (Exception &e) {
-        LogError("Can't add level %s as external level",
-                 m_xmdemo->levelFile().c_str());
-      }
-      m_PlaySpecificReplay = m_xmdemo->replayFile();
+      m_PlaySpecificReplay = loadDemoReplay(v_xmArgs.getOpt_demo_file());
     } catch (Exception &e) {
       LogError("Unable to load the demo file");
     }
@@ -624,6 +607,8 @@ void GameApp::run_load(int nNumArgs, char **ppcArgs) {
 #endif
   m_isODEInitialized = true;
 
+  LogInfo("Lua version: %s", LUA_RELEASE);
+
   /* load packs */
   LevelsManager::checkPrerequires();
 
@@ -675,31 +660,24 @@ void GameApp::run_load(int nNumArgs, char **ppcArgs) {
     /* What to do? */
     if (m_PlaySpecificLevelFile != "") {
       try {
-        LevelsManager::instance()->addExternalLevel(
+        m_PlaySpecificLevelId = LevelsManager::instance()->addExternalLevel(
           m_PlaySpecificLevelFile,
           xmDatabase::instance("main"),
           v_xmArgs.isOptServerOnly());
-        m_PlaySpecificLevelId = LevelsManager::instance()->LevelByFileName(
-          m_PlaySpecificLevelFile, xmDatabase::instance("main"));
+        if (m_PlaySpecificLevelId.empty()) {
+          m_PlaySpecificLevelId = LevelsManager::instance()->LevelByFileName(
+            m_PlaySpecificLevelFile, xmDatabase::instance("main"));
+        }
       } catch (Exception &e) {
         m_PlaySpecificLevelId = m_PlaySpecificLevelFile;
       }
     }
     if ((m_PlaySpecificLevelId != "")) {
       /* ======= PLAY SPECIFIC LEVEL ======= */
-
-      if (XMSession::instance()->clientGhostMode() == false &&
-          NetClient::instance()->isConnected()) {
-        StateManager::instance()->pushState(new StateWaitServerInstructions());
-      } else {
-        StateManager::instance()->pushState(
-          new StatePreplayingGame(m_PlaySpecificLevelId, false));
-      }
-      LogInfo("Playing as '%s'...", XMSession::instance()->profile().c_str());
+      playLevel(m_PlaySpecificLevelId);
     } else if (m_PlaySpecificReplay != "") {
       /* ======= PLAY SPECIFIC REPLAY ======= */
-      StateManager::instance()->pushState(
-        new StatePreplayingReplay(m_PlaySpecificReplay, false));
+      playReplay(m_PlaySpecificReplay);
     } else {
       /* display what must be displayed */
       StateManager::instance()->pushState(new StateMainMenu());
@@ -727,38 +705,52 @@ void GameApp::manageEvent(SDL_Event *Event) {
   std::string utf8Char;
 
   if (Event->type == SDL_KEYDOWN || Event->type == SDL_KEYUP) {
-    /* don't allow simple modifier key */
-    if (Event->key.keysym.sym == SDLK_RSHIFT ||
-        Event->key.keysym.sym == SDLK_LSHIFT ||
-        Event->key.keysym.sym == SDLK_RCTRL ||
-        Event->key.keysym.sym == SDLK_LCTRL ||
-        Event->key.keysym.sym == SDLK_RALT ||
-        Event->key.keysym.sym == SDLK_LALT ||
-        Event->key.keysym.sym == SDLK_RMETA ||
-        Event->key.keysym.sym == SDLK_LMETA) {
-      return;
+    /* ignore modifier-only key presses */
+    switch (Event->key.keysym.sym) {
+      case SDLK_RSHIFT:
+      case SDLK_LSHIFT:
+      case SDLK_RCTRL:
+      case SDLK_LCTRL:
+      case SDLK_RALT:
+      case SDLK_LALT:
+      case SDLK_RGUI:
+      case SDLK_LGUI:
+        return;
     }
   }
 
-  /* What event? */
+  SDL_StartTextInput();
+
   switch (Event->type) {
+    case SDL_TEXTINPUT:
+      utf8Char = Event->text.text;
+
+      StateManager::instance()->xmKey(
+        INPUT_TEXT,
+        XMKey(0, (SDL_Keymod)0, utf8Char));
+      break;
+
     case SDL_KEYDOWN:
-      utf8Char = unicode2utf8(Event->key.keysym.unicode);
-      // printf("%i - %i\n", Event->key.keysym.sym, utf8Char.c_str()[0]);
+      utf8Char = unicode2utf8(Event->key.keysym.sym);
+
       StateManager::instance()->xmKey(
         INPUT_DOWN,
-        XMKey(Event->key.keysym.sym, Event->key.keysym.mod, utf8Char));
+        XMKey(Event->key.keysym.sym, (SDL_Keymod)Event->key.keysym.mod, utf8Char, Event->key.repeat));
       break;
+
     case SDL_KEYUP:
-      utf8Char = unicode2utf8(Event->key.keysym.unicode);
+      utf8Char = unicode2utf8(Event->key.keysym.sym);
+
       StateManager::instance()->xmKey(
         INPUT_UP,
-        XMKey(Event->key.keysym.sym, Event->key.keysym.mod, utf8Char));
+        XMKey(Event->key.keysym.sym, (SDL_Keymod)Event->key.keysym.mod, utf8Char));
       break;
+
     case SDL_QUIT:
       /* Force quit */
       quit();
       break;
+
     case SDL_MOUSEBUTTONDOWN:
       /* Is this a double click? */
       getMousePos(&nX, &nY);
@@ -767,7 +759,7 @@ void GameApp::manageEvent(SDL_Event *Event) {
           (getXMTime() - fLastMouseClickTime) < MOUSE_DBCLICK_TIME) {
         /* Pass double click */
         StateManager::instance()->xmKey(INPUT_DOWN,
-                                        XMKey(Event->button.button, 2));
+                                        XMKey(Event->button.button, 1));
       } else {
         /* Pass ordinary click */
         StateManager::instance()->xmKey(INPUT_DOWN,
@@ -782,49 +774,108 @@ void GameApp::manageEvent(SDL_Event *Event) {
     case SDL_MOUSEBUTTONUP:
       StateManager::instance()->xmKey(INPUT_UP, XMKey(Event->button.button));
       break;
-
-    case SDL_JOYAXISMOTION:
-      StateManager::instance()->xmKey(
-        InputHandler::instance()->joystickAxisSens(Event->jaxis.value),
-        XMKey(InputHandler::instance()->getJoyId(Event->jbutton.which),
-              Event->jaxis.axis,
-              Event->jaxis.value));
+    case SDL_MOUSEWHEEL:
+      StateManager::instance()->xmKey(INPUT_SCROLL, XMKey(*Event));
       break;
-
-    case SDL_JOYBUTTONDOWN:
+    case SDL_CONTROLLERAXISMOTION:
       StateManager::instance()->xmKey(
-        INPUT_DOWN,
-        XMKey(InputHandler::instance()->getJoyId(Event->jbutton.which),
-              Event->jbutton.button));
+        Input::instance()->joystickAxisSens(Event->caxis.value),
+        XMKey(Input::instance()->getJoyById(Event->cbutton.which),
+              Event->caxis.axis,
+              Event->caxis.value));
       break;
-
-    case SDL_JOYBUTTONUP:
-      StateManager::instance()->xmKey(
-        INPUT_UP,
-        XMKey(InputHandler::instance()->getJoyId(Event->jbutton.which),
-              Event->jbutton.button));
-      break;
-
-    case SDL_ACTIVEEVENT:
-
-      if ((Event->active.state & SDL_APPMOUSEFOCUS) != 0) { // mouse focus
-        if (m_hasKeyboardFocus == false) {
-          StateManager::instance()->changeFocus(Event->active.gain == 1);
+    case SDL_USEREVENT: {
+      if (Event->user.code == SDL_CONTROLLERAXISMOTION) {
+        const auto &event = *(static_cast<JoyAxisEvent *>(Event->user.data1));
+        StateMenu *state = dynamic_cast<StateMenu *>(
+            StateManager::instance()->getTopState());
+        if (state) {
+          state->getGUI()->joystickAxisMotion(event);
         }
-        m_hasMouseFocus = (Event->active.gain == 1);
       }
+      break;
+    }
 
-      if ((Event->active.state & SDL_APPINPUTFOCUS) != 0) { // keyboard focus
-        if (m_hasMouseFocus == false) {
-          StateManager::instance()->changeFocus(Event->active.gain == 1);
+    case SDL_CONTROLLERBUTTONDOWN: {
+    case SDL_CONTROLLERBUTTONUP:
+      InputEventType type = Input::eventState(Event->type);
+      XMKey key = XMKey(
+          Input::instance()->getJoyById(Event->cbutton.which),
+          Event->cbutton.button);
+      StateManager::instance()->xmKey(type, key);
+      break;
+    }
+
+    case SDL_WINDOWEVENT: {
+      bool hasFocus = false;
+
+      switch (Event->window.event) {
+        case SDL_WINDOWEVENT_ENTER:
+        case SDL_WINDOWEVENT_LEAVE: {
+          switch (Event->window.event) {
+            case SDL_WINDOWEVENT_ENTER: hasFocus = true;  break;
+            case SDL_WINDOWEVENT_LEAVE: hasFocus = false; break;
+          }
+
+          if (!m_hasKeyboardFocus)
+            StateManager::instance()->changeFocus(hasFocus);
+
+          m_hasMouseFocus = hasFocus;
+          break;
         }
-        m_hasKeyboardFocus = (Event->active.gain == 1);
+        case SDL_WINDOWEVENT_FOCUS_GAINED: /* fall through */
+        case SDL_WINDOWEVENT_FOCUS_LOST: {
+          switch (Event->window.event) {
+            case SDL_WINDOWEVENT_FOCUS_GAINED: hasFocus = true;  break;
+            case SDL_WINDOWEVENT_FOCUS_LOST:   hasFocus = false; break;
+          }
+
+          if (hasFocus) {
+            // Pending keydown events need to be flushed after gaining focus
+            // because SDL2 will send them for keys that were pressed outside the game
+            // (e.g. when alt-tabbing in)
+            SDL_FlushEvents(SDL_KEYDOWN, SDL_KEYDOWN);
+          } else {
+            /*
+             * With SDL2, input events come in in the opposite order from SDL1.2
+             * (the order used to be: "focus lost" -> "key released").
+             * We need to invalidate the keys here so they don't persist after focus is lost
+             */
+            StatePlayingLocal *state = dynamic_cast<StatePlayingLocal *>(
+                StateManager::instance()->getTopState());
+            if (state)
+              state->dealWithActivedKeys();
+          }
+
+          if (!m_hasMouseFocus)
+            StateManager::instance()->changeFocus(hasFocus);
+
+          m_hasKeyboardFocus = hasFocus;
+          break;
+        }
+        case SDL_WINDOWEVENT_EXPOSED:
+          StateManager::instance()->setInvalidated(true);
+          break;
+        case SDL_WINDOWEVENT_SHOWN:
+          StateManager::instance()->changeVisibility(true);
+          m_isIconified = false;
+          break;
+        case SDL_WINDOWEVENT_HIDDEN:
+          StateManager::instance()->changeVisibility(false);
+          m_isIconified = true;
+          break;
       }
 
-      if ((Event->active.state & SDL_APPACTIVE) != 0) {
-        StateManager::instance()->changeVisibility(Event->active.gain == 1);
-        m_isIconified = (Event->active.gain == 0);
-      }
+      break;
+    }
+    case SDL_DROPFILE: {
+      char *file = Event->drop.file;
+      std::string path(file);
+      SDL_free(file);
+      StateManager::instance()->fileDrop(path);
+
+      break;
+    }
   }
 }
 
@@ -950,8 +1001,8 @@ void GameApp::run_unload() {
     delete m_pWebLevels;
   }
 
-  if (InputHandler::instance() != NULL) {
-    InputHandler::instance()
+  if (Input::instance() != NULL) {
+    Input::instance()
       ->uninit(); // uinit the input, but you can still save the config
   }
 
@@ -978,12 +1029,13 @@ void GameApp::run_unload() {
   if (Logger::isInitialized()) {
     LogDebug("UserUnload saveConfig at %.3f", GameApp::getXMTime());
   }
-  if (drawLib != NULL) { /* save config only if drawLib was initialized */
+  auto dbInstance = xmDatabase::instance("main");
+  if (drawLib != NULL && dbInstance->isOpen()) { /* save config only if drawLib was initialized */
     XMSession::instance("file")->save(m_userConfig,
                                       xmDatabase::instance("main"));
-    InputHandler::instance()->saveConfig(
+    Input::instance()->saveConfig(
       m_userConfig,
-      xmDatabase::instance("main"),
+      dbInstance,
       XMSession::instance("file")->profile());
     m_userConfig->saveFile();
   }
@@ -996,7 +1048,7 @@ void GameApp::run_unload() {
     drawLib->unInit();
   }
 
-  InputHandler::destroy();
+  Input::destroy();
   LevelsManager::destroy();
   GeomsManager::destroy();
   Theme::destroy();
